/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_INVESTIGATION__
#define __INVESTIGATION_DONNEES_INVESTIGATION__

#include <stack>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(InvestigationSite); } }
#include <src/investigation/donnees/arbreinvestigation.h>
#include <src/investigation/donnees/compteurs.h>

namespace Investigation
{
	namespace Donnees
	{
		class SiteTropComplexe : public Investigation::Global::Exception::Execution
		{
			public:
				SiteTropComplexe()
				:Execution(SIGSEGV,"Site too complex. Try on a more powerfull machine...") {};
		};

		struct InvestigationSite
		{
			CompteursSite operator()(const SiteSP& site, const std::string& mot_clef_requetes) const
			{
				try
				{
					std::cerr << "Site investigation... Please be patient!" << std::endl;
					Requete requete;
					size_t essais = 0;
					ArbreInvestigationSP arbre(new ArbreInvestigation(requete));
					std::stack<ArbreInvestigationSP> pile_requetes;
					pile_requetes.push(arbre);
					std::ostringstream requetes_loguees;
					if(not mot_clef_requetes.empty())
					{
						requetes_loguees << "Requests ending with " << mot_clef_requetes << ": (continue means that the investigation continues with this request as prefix, stop means that the investigation will stop on this leaf)" << std::endl << std::endl;
					}
					while(not pile_requetes.empty())
					{
						ArbreInvestigationSP noeud_courant = pile_requetes.top();
						pile_requetes.pop();
						std::cerr << "  Request: " << (++essais) << " Still on stack: " << pile_requetes.size() << "                     \r";
						SiteSP site_courant(new Site(site->clone()));
						auto mots_clef= (*site_courant)[noeud_courant->_requete];
						if(not ((*noeud_courant)%(site_courant)))
						{
							for(auto it=mots_clef.rbegin() ; it!=mots_clef.rend() ; ++it)
							{
								Requete itreq = noeud_courant->_requete;
								itreq = itreq << (*it);
								pile_requetes.push((*noeud_courant) += itreq); 
							}
							if(not mot_clef_requetes.empty() and noeud_courant->_requete and (mot_clef_requetes==static_cast<std::string>(noeud_courant->_requete)))
							{
								requetes_loguees << "Request" << noeud_courant->_requete << ": continue" << std::endl;
							}
						}
						else
						{
							if(not mot_clef_requetes.empty() and noeud_courant->_requete and (mot_clef_requetes==static_cast<std::string>(noeud_courant->_requete)))
							{
								requetes_loguees << "Request" << noeud_courant->_requete << ": stop" << std::endl;
							}
						}
						noeud_courant->_site = site_courant;
					}
					std::cout << "  Requests: " << essais << "                                                                    " << std::endl;
					std::cout << std::endl << requetes_loguees.str() << std::endl;
					return arbre->compte();
				}
				catch(std::bad_alloc& e)
				{
					throw SiteTropComplexe();
				}
			}
		};
	}
}
#endif
