/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_REPONSECHOIX__
#define __INVESTIGATION_DONNEES_REPONSECHOIX__

#include <vector>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(ReponseChoix); } }
#include <src/investigation/donnees/question.h>
#include <src/investigation/donnees/html.h>

namespace Investigation
{
	namespace Donnees
	{
		class CompteursSite;
		class LienInvalide : public Investigation::Global::Exception::Compilation
		{
			public:
				LienInvalide(const Investigation::Global::Source::PositionSP& position, const std::string& identifiant)
				:Compilation(position,std::string("Link to pages ")+identifiant+" invalid") {};
		};

		class ReponseChoix : public Reponse
		{
			public:
				ReponseChoix(const std::string& identifiant, const std::string& intitule, const Investigation::Global::Source::PositionSP& position);
				virtual ~ReponseChoix();
				virtual Html::SequenceLi operator() (const std::string& methode, const Investigation::Donnees::Requete& requete, Investigation::Donnees::Liens& liens) const;
				virtual ReponseSP clone() const;
				virtual std::set<std::string> operator[] (const Investigation::Donnees::Requete& requete);
				virtual bool operator%(const Reponse& reponse) const;
				bool operator%(const ReponseChoix& reponsechoix) const;
				virtual void liens_valides(const std::set<std::string>& identifiants) const;
				virtual void propage_infos(const std::string& page, const std::string& question);
				virtual void compte(CompteursSite& compteurs) const;
				virtual IdentifiantPagesActives pages_actives_source(const Identifiant& page) const;
			private:
				std::string _identifiant;
				std::string _intitule;
				std::string _question;
				std::string _page;
				Investigation::Global::Source::PositionSP _position;
		};
	}
}
#endif
