/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#include <src/investigation/donnees/selectionpage.h>

using namespace Investigation::Donnees;

SelectionPage::SelectionPage(const Selection<Instruction>& selection, const Investigation::Global::Source::PositionSP& position)
	:Instruction(position), Selection<Instruction>(selection) {}
SelectionPage::SelectionPage(const typename Selection<Instruction>::SequenceClauses& clauses, const SequenceInstructions& clause_par_defaut, const Investigation::Global::Source::PositionSP& position)
	:Instruction(position), Selection(clauses,clause_par_defaut) { }
SelectionPage::~SelectionPage() {}
SequencePages SelectionPage::operator() (const Requete& requete) const
{
	SequencePages resultat;
	SequenceInstructions clause = (static_cast<Selection<Instruction> >(*this))(requete);
	for(auto it=clause.cbegin() ; it!=clause.cend() ; ++it)
	{
		SequencePages sous_resultat = (**it)(requete);
		resultat.insert(resultat.end(),sous_resultat.begin(),sous_resultat.end());
	}
	return resultat;
}
InstructionSP SelectionPage::clone() const
{
	SelectionPageSP instruction(new SelectionPage(Selection<Instruction>::clone_interne(),_position));
	return instruction;
}
std::set<std::string> SelectionPage::operator[] (const Requete& requete)
{
	return Selection<Instruction>::operator[](requete);
}
bool SelectionPage::operator%(const Instruction& instruction) const
{
	const SelectionPage& page=dynamic_cast<const SelectionPage&>(instruction); // throw bad_cast
	return (*this)%(page);
}
bool SelectionPage::operator%(const SelectionPage& page) const
{
	return Selection<Instruction>::operator%(page);
}

std::set<std::string> SelectionPage::identifiants() const
{
	std::set<std::string> resultat;
	std::vector<InstructionSP> instructions = (*this);
	for(auto it=instructions.cbegin() ; it!=instructions.cend() ; ++it)
	{
		std::set<std::string> sous_resultat = (*it)->identifiants();
		resultat.insert(sous_resultat.cbegin(),sous_resultat.cend());
	}
	return resultat;
}
void SelectionPage::liens_valides(const std::set<std::string>& identifiants) const
{
	std::vector<InstructionSP> instructions = (*this);
	for(auto it=instructions.cbegin() ; it!=instructions.cend() ; ++it)
	{
		(*it)->liens_valides(identifiants);
	}

}
void SelectionPage::propage_infos()
{
	std::vector<InstructionSP> instructions = (*this);
	for(auto it=instructions.cbegin() ; it!=instructions.cend() ; ++it)
	{
		(*it)->propage_infos();
	}
}
void SelectionPage::compte(CompteursSite& compteurs) const
{
	Selection<Instruction>::compte(compteurs);
}

IdentifiantPagesActives SelectionPage::pages_actives_destination() const
{
	IdentifiantPagesActives id;
	SequenceInstructions clause = (static_cast<Selection<Instruction> >(*this))();
	for(auto it=clause.cbegin() ; it!=clause.cend() ; ++it)
	{
		id += (*it)->pages_actives_destination();
	}
	return id;
}

IdentifiantPagesActives SelectionPage::pages_actives_source() const
{
	IdentifiantPagesActives id;
	SequenceInstructions clause = (static_cast<Selection<Instruction> >(*this))();
	for(auto it=clause.cbegin() ; it!=clause.cend() ; ++it)
	{
		id += (*it)->pages_actives_source();
	}
	return id;
}
