/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_SITE__
#define __INVESTIGATION_DONNEES_SITE__

#include <map>
#include <set>
#include <string>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(Site); } }

#include <src/investigation/donnees/instruction.h>

namespace Investigation
{
	namespace Donnees
	{
		class Serveur;
		DECL_SHARED_PTR(Page);
		class Requete;
		class CompteursSite;
		namespace Html { class Html; }
		class Liens;
		class IdentifiantPagesActives;
		class Site
		{
			friend class Serveur;
			public:
				Site(const std::string& titre);
				void operator<<(const std::string& style);
				void operator<<(const PageSP& page_de_demarrage);
				void operator<<(const SequenceInstructions& instructions);
				SequencePages operator()(const Requete& requete) const;
				std::set<std::string> operator[] (const Requete& requete);
				Site clone() const;
				bool operator%(const Site& site) const;
				void compte(CompteursSite& compteurs) const;
				void integrite();
				Html::Html operator() (const SequencePages& pages, const std::string& methode, const Requete& requete, Liens& liens) const;
				IdentifiantPagesActives pages_actives_destination() const;
				IdentifiantPagesActives pages_actives_source() const;
			private:
				const std::string _titre;
				PageSP _page_de_demarrage;
				SequenceInstructions _instructions;
				std::string _style;
				
		};
	}
}
#endif
